/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 2012 Texas Instruments Incorporated                |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/
/**
 * @file        sl_selftest.c
 * @version     $$file_version$$
 * @brief       File contains esm handling and interrupt handlers for esm errors
 *
 * @details This file implements interrupt handlers for the esmHigh and esmLow interrupts\n
 * the esm group 2 and group 1 handlers have been implemented which can be further extended\n
 * depending on the end application.It provides the Init function which updates the application\n
 * callback function and the interrupt handlers for esmHigh and esmLow interrupt.It also provides\n
 * API's which provide the information on the nerror pin status
 */
#include <sl_api.h>
#include <sl_priv.h>
#if FUNCTION_PROFILING_ENABLED
#include "sl_sys_pmu.h"
#endif

static
void sl_esm_high_intr_handler(void);
static
void sl_esm_low_intr_handler(void);
static
void esmGroup1Handler(uint8 esmChannel);
static
void esmGroup2Handler(uint8 esmChannel);

/*SAFETYMCUSW 298 S MR:- <APPROVED> "Reason -  Needed"*/
static ESM_Application_Callback ESM_AppCallback;
static uint32 callbackCancelCount = 0u;

/*SAFETYMCUSW 61 D MR: 8.10,8.11 <APPROVED> "This function will be called by application so not static" */
void SL_ESM_Init(ESM_Application_Callback appCallback)
{
	vimRAM->ISR[1]  = &sl_esm_high_intr_handler;
	vimRAM->ISR[21] = &sl_esm_low_intr_handler;
	ESM_AppCallback = appCallback;
}

boolean SL_ESM_nERROR_Active(void)
{
    boolean retval = FALSE;
    if(ESM_ERROR_PIN_OFF== BF_GET(sl_esmREG->EPSR,BF_EPSF_START,BF_EPSF_LENGTH)) {
        retval = FALSE;
    } else {
        retval = TRUE;
    }
	return(retval);
}
/*SAFETYMCUSW 69 S MR:3.4 <APPROVED> "Reason -  used for creation of code section for interrupt handler"*/
#pragma CODE_STATE(sl_esm_high_intr_handler, 32)
/*SAFETYMCUSW 69 S MR:3.4 <APPROVED> "Reason -  used for creation of code section for interrupt handler"*/
#pragma INTERRUPT(sl_esm_high_intr_handler, FIQ)
static void sl_esm_high_intr_handler(void)
{
    register uint8  esmOffH,esmChannel;
#if FUNCTION_PROFILING_ENABLED
    register uint32 entrytick=0u;
    entrytick = _sl_pmuGetCycleCount_();
#endif
    esmOffH = (uint8) sl_esmREG->IOFFHR;
    if (0u != esmOffH) {
        esmOffH--;
        if (esmOffH >= 96u) {
        	esmChannel = esmOffH - 64u;
        	esmGroup2Handler(esmChannel);
        } else if (esmOffH >= 64u) {
        	esmChannel = esmOffH - 32u;
        	esmGroup1Handler(esmChannel);
        } else if (esmOffH >= 32u) {
        	esmChannel = esmOffH - 32u;
        	esmGroup2Handler(esmChannel);
        } else if (esmOffH > 0u) {
        	esmChannel = esmOffH;
        	esmGroup1Handler(esmChannel);
        }
        else {
			/* Misra C fix 477S added to make the else part non empty */
			esmOffH++;
                /* TODO: Shouldn't be here... */
        }
    }
    else{
		/* Misra C fix 477S added to make the else part non empty */
    	esmOffH++;
    }/* Shouldn't get here... */
#if FUNCTION_PROFILING_ENABLED
    if ((esmChannel != ESM_G2ERR_B0TCM_ADDPAR) &&
    		(esmChannel != ESM_G2ERR_B1TCM_ADDPAR)) {
    	if(SL_Profile_Struct[SL_Active_Profile_Testtype-TESTTYPE_MIN].esm_entrytick == 0U )
    	{
        SL_Profile_Struct[SL_Active_Profile_Testtype-TESTTYPE_MIN].esm_entrytick = entrytick;
        SL_Profile_Struct[SL_Active_Profile_Testtype-TESTTYPE_MIN].esm_exittick = _sl_pmuGetCycleCount_();
    	}
    }
#endif
}
/*SAFETYMCUSW 69 S MR:3.4 <APPROVED> "Reason -  used for creation of code section for interrupt handler"*/
#pragma CODE_STATE(sl_esm_low_intr_handler, 32)
/*SAFETYMCUSW 69 S MR:3.4 <APPROVED> "Reason -  used for creation of code section for interrupt handler"*/
#pragma INTERRUPT(sl_esm_low_intr_handler, FIQ)
static void sl_esm_low_intr_handler(void) {
    register uint8  esmOffL,esmChannel;
#if FUNCTION_PROFILING_ENABLED
    register uint32 entrytick=0u;
    entrytick = _sl_pmuGetCycleCount_();
#endif

    esmOffL = ((uint8)sl_esmREG->IOFFLR - (uint8)1u);
    if ((esmOffL >= 64U) && (esmOffL < 96U))
    {
    	esmChannel = esmOffL-32;
    	esmGroup1Handler(esmChannel);
    }else if( esmOffL < 32U){
    	esmChannel = esmOffL;
    	esmGroup1Handler(esmChannel);
    }
    else {
		/* Misra C fix 477S added to make the else part non empty */
    	esmOffL++;
            /* TODO: Shouldn't be here... */
    }
#if FUNCTION_PROFILING_ENABLED
   	if(SL_Profile_Struct[SL_Active_Profile_Testtype-TESTTYPE_MIN].esm_entrytick == 0U )
    	{
        SL_Profile_Struct[SL_Active_Profile_Testtype-TESTTYPE_MIN].esm_entrytick = entrytick;
        SL_Profile_Struct[SL_Active_Profile_Testtype-TESTTYPE_MIN].esm_exittick = _sl_pmuGetCycleCount_();
    	}
#endif
    return;
}


static void esmGroup2Handler (uint8 esmChannel)
{
	/*SAFETYMCUSW 331 S MR:10.1 <APPROVED> ""*/
	/*SAFETYMCUSW 434 S MR:10.1 <APPROVED> ""*/
    boolean cancelCallback = FALSE;
    register uint32 callbkParam1 = 0u, callbkParam2 = 0u, callbkParam3 = 0u;

    /* No need to clear ESM bit since reading ESMOFFH automatically clears the status bit */
    switch (esmChannel) {
        case ESM_G2ERR_CCMR4_COMP:
            /* No additional parameters available */
        	callbkParam1 = 0u;
            /* Clear the module error */
            ccmr4fREG1->_CCMSR = CCMR4F_CCMSR_CMPE;
            /* Cancel callback if this was intentional */
            if (TRUE == SL_FLAG_GET(CCMR4F_ERROR_FORCING_TEST)) {
            	callbackCancelCount++;
            	cancelCallback = TRUE;
            }
        break;

        case ESM_G2ERR_FMC_UNCORR:
            if (TRUE == SL_FLAG_GET(FLASH_ADDRESS_PARITY_SELF_TEST)) {
            	callbackCancelCount++;
            	cancelCallback = TRUE;
            }
            /* Flash Un-correctable error, capture the error address */
        	callbkParam1 = sl_flashWREG->FUNCERRADD;
            /* No status to clear here */
        break;

        case ESM_G2ERR_B0TCM_UNCORR:
            if (TRUE == SL_FLAG_GET(SRAM_RADECODE_DIAGNOSTICS)) {
            	callbackCancelCount++;
            	cancelCallback = TRUE;
            }
        	callbkParam1 = sl_tcram1REG->RAMUERRADDR;
            /* No status to clear here */
        break;

        case ESM_G2ERR_B1TCM_UNCORR:
            if (TRUE == SL_FLAG_GET(SRAM_RADECODE_DIAGNOSTICS)) {
            	callbackCancelCount++;
            	cancelCallback = TRUE;
            }
        	callbkParam1 = sl_tcram2REG->RAMUERRADDR;
            /* No status to clear here */
        break;

        case ESM_G2ERR_B0TCM_ADDPAR:
        	callbkParam1 = sl_tcram1REG->RAMPERADDR;
            callbkParam2 = (sl_tcram1REG->RAMERRSTATUS
                                & (TCRAM_RAMERRSTATUS_WADDR_PAR_FAIL | TCRAM_RAMERRSTATUS_WADDR_PAR_FAIL));
            callbkParam3 = (uint32) 0u;
            /* Clear the WADDR/RADDR PAR Fail bits in TCM Regs */
            break;

        case ESM_G2ERR_B1TCM_ADDPAR:
        	callbkParam1 = sl_tcram2REG->RAMPERADDR;
            /* No status to clear here */
            break;

        case ESM_G2ERR_FLASH_LIVLCK:
        	callbkParam1 = 0u;
            /* No status to clear here */
            break;

        case ESM_G2ERR_RTI_WWD_NMI:
        	callbkParam1 = 0u;
            /* No status to clear here */
        break;
        default:
            break;
    }
    /* Call application callback only if it's registered & if it's not masked */
	/* the MSB of contains information regarding group and LSB contains the esm channel number*/
    if ((0u != ESM_AppCallback) && ((FALSE) == cancelCallback)) {
    	(*(ESM_AppCallback))((uint32)(ESM_GRP2_MASK | esmChannel), callbkParam1, callbkParam2, callbkParam3);
    }
    return;
}

static
void esmGroup1Handler(uint8 esmChannel)
{
	/*SAFETYMCUSW 331 S MR:10.1 <APPROVED> ""*/
	/*SAFETYMCUSW 434 S MR:10.1 <APPROVED> ""*/
	boolean cancelCallback = FALSE;
    register uint32 callbkParam1 = 0u, callbkParam2 = 0u, callbkParam3 = 0u;

	switch (esmChannel) {
        /** F021 Flash Correctabe error (bus1 & bus2) & non-EEPROM **/
		case ESM_G1ERR_FMC_CORR:
			/* Retrieve the callback parameters */
			callbkParam1 = (sl_flashWREG->FCORERRCNT & 0x0000FFFFu);
			callbkParam2 = (sl_flashWREG->FCORERRADD);
			callbkParam3 = (sl_flashWREG->FCORERRPOS & 0x000003FFu);
			break;

        /** PLL1 Slip error **/
        case ESM_G1ERR_PLL1_SLIP:
            /** Store the callback parameters & reset the status registers */
            callbkParam1 = (uint32)((sl_systemREG1->GBLSTAT & 0x00000300u) >> 8u);
            sl_systemREG1->GBLSTAT |= (sl_systemREG1->GBLSTAT & 0x00000300u);
            break;

        /** LPOCLK Detect error */
        case ESM_G1ERR_CLOCKMON_INTR: /* \todo Need to confirm the source & params */
            break;

        /** VIM Parity Error **/
        case ESM_G1ERR_VIMRAM_PARITY:
            /* Extract the parameters & clear the flag */
            callbkParam1 = SL_VIM_ADDERR;
            SL_VIM_PARFLG = 1u;
            break;

        /** B0TCM ECC Correctable error **/
        case ESM_G1ERR_B0TCM_CORRERR:
            /* Call application if this error was _not_ intentional (self-test) */
        	callbkParam1 = (sl_tcram1REG->RAMSERRADDR);
            break;

        /** B1TCM ECC Correctable error **/
        case ESM_G1ERR_B1TCM_CORRERR:
        	callbkParam1 = (sl_tcram2REG->RAMSERRADDR);
            /* Call application if this error was _not_ intentional (self-test) */
            break;

        /** DCC Error */
        case ESM_G1ERR_DCC1_ERR:
            callbkParam1 = sl_dccREG1->CNT0;
            callbkParam2 = sl_dccREG1->CNT1;
            sl_dccREG1->STAT |= DCC_DCCSTAT_ERR_FLG;
            break;

        /** CCMR4 Self-Test Compare error */
        case ESM_G1ERR_CCMR4_SELFTEST:
			/*ccmr4f selftests have has synchronous and asynchronous modes,the efuse esm errors are not blocked by disabling interrupt during selftest*/
		    /*blocking the esm application call back if the interrupt was due to a selftest*/
            if (TRUE == SL_FLAG_GET(CCMR4F_ERROR_FORCING_TEST)) {
            	callbackCancelCount++;
            	cancelCallback = TRUE;
            }
        	break;

        case ESM_G1ERR_EEPROM_CORRERR:
			/* Retrieve the callback parameters */
			callbkParam1 = (sl_flashWREG->EECORRERRADD);
			callbkParam2 = (sl_flashWREG->EECORRERRCNT);
			callbkParam3 = (sl_flashWREG->EECORRERRPOS & 0x000001FFu);
            /* \todo */
        	break;

        case ESM_G1ERR_EEPROM_UNCORRERR:
        	callbkParam1 = (sl_flashWREG->EEUNCERRADD);
        	callbkParam1 = (sl_flashWREG->EESTATUS);
            /* \todo */
        	break;
#if defined(_TMS570LS31x_) || defined(_TMS570LS12x_) || defined(_RM48x_) || defined(_RM46x_)
		case ESM_G1ERR_PSCON_COMPARE_ERR:
     		/*pscon selftests have has synchronous and asynchronous modes,the efuse esm errors are not blocked by disabling interrupt during selftest*/
		    /*blocking the esm application call back if the interrupt was due to a selftest*/
			if ((PMM_PRCKEYREG_MKEY_ERROR_FORCING_MODE == sl_pmmREG->PRCKEYREG) && (TRUE == SL_FLAG_GET(PSCON_ERROR_FORCING))) {
				/*SAFETYMCUSW 331 S MR:10.1 <APPROVED> ""*/
				/*SAFETYMCUSW 434 S MR:10.1 <APPROVED> ""*/
				cancelCallback = TRUE;
			} else {
				/*SAFETYMCUSW 331 S MR:10.1 <APPROVED> ""*/
				/*SAFETYMCUSW 434 S MR:10.1 <APPROVED> ""*/
				cancelCallback = FALSE;
			}
			callbkParam1 = (((sl_pmmREG->LPDDCSTAT1 & PMM_PSCON_LPDDCSTAT1_LCMPE) >> 16)
								| (sl_pmmREG->MPDDCSTAT1 & PMM_PSCON_MPDDCSTAT1_MCMPE));
			break;

		case ESM_G1ERR_PSCON_SELTEST_ERR:
			/*pscon selftests have has synchronous and asynchronous modes,the efuse esm errors are not blocked by disabling interrupt during selftest*/
		    /*blocking the esm application call back if the interrupt was due to a selftest*/
			if (TRUE == SL_FLAG_GET(PSCON_SELF_TEST)){
				/* Since this is a self test related error, it's always intentionally
				 * created by kicking off the ST in PRCKEYREG. Selftest API takes
				 * care of clearing this error nothing needs to be done here */
				/*SAFETYMCUSW 331 S MR:10.1 <APPROVED> ""*/
				/*SAFETYMCUSW 434 S MR:10.1 <APPROVED> ""*/
				cancelCallback  = TRUE;
				callbackCancelCount++;
				SL_FLAG_CLEAR(PSCON_SELF_TEST);
			}
			break;
#endif
		case ESM_G1ERR_EFUSE_SELTEST_ERR:
			/*efuse selftests have synchronous and asynchronous modes.the efuse esm errors are not blocked by disabling interrupt during selftest*/
			if ((TRUE ==  SL_FLAG_GET(EFUSE_SELF_TEST_STUCK_AT_ZERO))||(TRUE ==  SL_FLAG_GET(EFUSE_SELF_TEST_ECC))){
				/* Since this is a self test related error, it's always intentionally
				 * created by kicking off the ST in PRCKEYREG. Selftest API takes
				 * care of clearing this error nothing needs to be done here */
				/*SAFETYMCUSW 331 S MR:10.1 <APPROVED> ""*/
				/*SAFETYMCUSW 434 S MR:10.1 <APPROVED> ""*/
				cancelCallback  = TRUE;
				callbackCancelCount++;
				SL_FLAG_CLEAR(EFUSE_SELF_TEST_ECC);
				SL_FLAG_CLEAR(EFUSE_SELF_TEST_STUCK_AT_ZERO);
			}
			break;

		case ESM_G1ERR_EFUSE_ERR_OTHER:
			/*efuse has synchronous and asynchronous modes,the efuse esm errors are not blocked by disabling interrupt during selftest*/
		    /*blocking the esm application call back if the interrupt was due to a selftest*/
			if (TRUE ==  SL_FLAG_GET(EFUSE_SELF_TEST_ECC)){
				/* Since this is a self test related error, it's always intentionally
				 * created by kicking off the ST in PRCKEYREG. Selftest API takes
				 * care of clearing this error nothing needs to be done here */
				/*SAFETYMCUSW 331 S MR:10.1 <APPROVED> ""*/
				/*SAFETYMCUSW 434 S MR:10.1 <APPROVED> ""*/
				cancelCallback  = TRUE;
				callbackCancelCount++;
				SL_FLAG_CLEAR(EFUSE_SELF_TEST_ECC);
			}
			break;

        /** PLL2 Slip error **/
        case ESM_G1ERR_PLL2_SLIP:
            /** \todo Looks like no slip info is available. Double check */
        	break;

        /** DCC Error */
        case ESM_G1ERR_DCC2_ERR:
            callbkParam1 = sl_dccREG2->CNT0;
            callbkParam2 = sl_dccREG2->CNT1;
            sl_dccREG2->STAT |= DCC_DCCSTAT_ERR_FLG;
            break;

		default:
			break;
	}
    /* Call application callback only if it's registered & if it's not masked */
	/* the MSB of contains information regarding group and LSB contains the esm channel number*/
    if ((0u != ESM_AppCallback) && ((FALSE) == cancelCallback)) {
        (*(ESM_AppCallback))((uint32)(ESM_GRP1_MASK|esmChannel), callbkParam1, callbkParam2, callbkParam3);
    }
return;
}
